%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   Please check our website for the latest version of this script:
%   www.cs.ubc.ca/labs/imager/tr/2010/TheoryOfPlenopticMultiplexing/
%
%   This demo reconstructs (demosaicks) a RAW sensor image in the Fourier 
%   domain. 
%
%   Ivo Ihrke [ivo.ihrke@cs.ubc.ca]
%   Gordon Wetzstein [wetzste1@cs.ubc.ca]
%   PSM Lab, UBC
%   March 2010
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;

I = double( imread ( '../data/bayer_dc.pgm' ) );

Ifft = fftshift(fft2(I));

tx = size(I,2)/4;
ty = size(I,1)/4;

%lower limit
llx=1;
lly=1;

%upper limit
ulx=size(I,2);
uly=size(I,1);

%shift in FT due to even #scanlines and repetitions of bayer pattern
shiftx=0;
shifty=0;


%middle tile coord's
mty = ((ty+1):(3*ty))+shifty;
mtx = ((tx+1):(3*tx))+shiftx;

%lower range
ltx = llx:(mtx(1)-1);
lty = lly:(mty(1)-1);

%upper range
utx = (mtx(length(mtx))+1):ulx;
uty = (mty(length(mty))+1):uly;


%this is for the smaller versions
rtxl = 1:(tx-shiftx);
rtxu = ((tx+1)-shiftx):2*tx;

rtyl = 1:(ty-shifty);
rtyu = ((ty+1)-shifty):2*ty;


Ifft_lr = Ifft( mty+1, mtx );

Ifft_ll( :, rtxu )     = Ifft( mty, ltx );
Ifft_ll( :, rtxl )     = Ifft( mty, utx );

Ifft_ur(  rtyu, : )   = Ifft( lty , mtx );
Ifft_ur(  rtyl, : )   = Ifft( uty , mtx );

%Ifft_green( (ty+1):2*ty, : )     = Ifft( 1:ty, (tx+1):(3*tx) );
%Ifft_green( 1:ty, : )            = Ifft( (3*ty+1):(4*ty), (tx+1):(3*tx));

Ifft_ul( rtyl, rtxl )          = Ifft( uty, utx );
Ifft_ul( rtyu, rtxl )          = Ifft( lty, utx );
Ifft_ul( rtyl, rtxu )          = Ifft( uty, ltx );
Ifft_ul( rtyu, rtxu )          = Ifft( lty, ltx);

%Ifft_red   = 0.25*Ifft_lr  -0.25*Ifft_ul - 0.25*Ifft_ll + 0.25*Ifft_ur; 
%Ifft_green = 0.25*Ifft_lr  +0.25*Ifft_ul; 
%Ifft_blue  = 0.25*Ifft_lr  -0.25*Ifft_ul + 0.25*Ifft_ll - 0.25*Ifft_ur; 
%Ifft_gray  = 0.25*Ifft_lr;

Ifft_lr = -circshift(Ifft_lr,[1  0]);
Ifft_red   = 0.25*Ifft_lr  +0.25*Ifft_ul + 0.5*Ifft_ll; 
Ifft_green = 0.25*Ifft_lr  -0.25*Ifft_ul; 
Ifft_blue  = 0.25*Ifft_lr  +0.25*Ifft_ul - 0.5*Ifft_ll; 
Ifft_gray  = 0.25*Ifft_lr;


I_gray  = abs((ifft2(ifftshift( Ifft_gray  ))) / ( max( I(:) )));
I_red   = abs((ifft2(ifftshift( Ifft_red   ))) / ( max( I(:) )));
I_blue  = abs((ifft2(ifftshift( Ifft_blue  ))) / ( max( I(:) )));
I_green = abs((ifft2(ifftshift( Ifft_green ))) / ( max( I(:) )));

subplot(1,4,1)
imshow( I_red)
%imshow( abs(Ifft_red).^(1/2.2)/3000 ) ;
imshow( abs(ifft2(ifftshift(Ifft_ur))) / ( 1 * max( I (:)))  );
title('Demosaicked red channel');

subplot(1,4,2);
imshow( I_green ); 
%imshow( abs(Ifft_blue).^(1/2.2)/3000 ) ;
imshow( abs(ifft2(ifftshift(Ifft_ll))) / ( 1 * max( I (:)))  );
title('Demosaicked green channel');

subplot(1,4,3);
imshow( I_blue ); 
imshow( abs(ifft2(ifftshift(Ifft_lr))) / ( 4 * max( I (:)))  );
title('Demosaicked blue channel');

subplot(1,4,4);
Irgb(:,:,1)=I_red;
Irgb(:,:,2)=I_green;
Irgb(:,:,3)=I_blue;
imshow(Irgb)
title('Fourier-Demosaicked Image');

imwrite(Irgb, 'bayer_fourier.png');

%imperx image

%mean = 2*green + 1*red + 1*blue    (middle peak) (lower right)
%      -1*red   +2*green -1*blue      (upper left corner) 
%      -1*red +  1*blue              (lower left corner)
%       1*red + -1*blue              (upper right corner)

