%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   Please check our website for the latest version of this script:
%   www.cs.ubc.ca/labs/imager/tr/2010/TheoryOfPlenopticMultiplexing/
%  
% general fourier reconstruction 
% applied to dappled photography 
% dataset 
%
% (c) Ivo Ihrke
%     University of British Columbia 
%
%  2009-04-21
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all

filename = '../data/InputCones_9x9.png';

% check if file exists
if exist(filename, 'file') == 0
	error(['File ' filename ' does not exist. Make sure to follow the instructions in ../README!']);
end

%input image
I=double(imread(filename));

%dimensions
sx = 238;
sy = 191;

num_tiles = 9;

disp('computing 2D FFT')
for c=1:size(I,3)
  Ifft(:,:,c) = fftshift(fft2(I(:,:,c)));
end

disp('assembling fourier tiles');
Ifftc = zeros( sy, sx, size(I,3), num_tiles^2 );

offx = [ (1:sx:size(I,2)) size(I,2) ];
offy = [ (1:sy:size(I,1)) size(I,1) ];


for x=1:num_tiles-1
  for y=1:num_tiles-1
    off = (y-1) * num_tiles + x;
    
    Ifftc ( :,:,:,off ) = Ifft( offy(y):(offy(y+1)-1) , offx(x): ...
				(offx(x+1)-1), : );
  end
end


disp('assembling correlation matrix');

%assemble de-correlation matrix
F = zeros( num_tiles^2, num_tiles^2 );

for y=1:num_tiles
  for x=1:num_tiles
    off = (y-1)*num_tiles+x;
    dummy=zeros(num_tiles, num_tiles );
    dummy(x,y) = 1;
    dummy = ifft2(ifftshift(dummy));
    Finv( off, : ) = dummy(:) * num_tiles;
  end
end
F=Finv';

W = eye(num_tiles^2);
W(41,41) = 20;

PI = F;
SIGMA_hat = F * (Finv * W);

M_hat_inv = pinv( PI' ) * pinv ( SIGMA_hat ); 

disp('de-correlating fft tiles');

% this is the conceptual loop
%Ic_up_dec = zeros( size ( Ic_up ) );
%for x=1:size(Ic_up,2)
%  for y=1:size(Ic_up,1)
%    for c=1:size(I,3)
%      Ic_up_dec( y, x, c, : ) = Minv * squeeze(Ic_up(y,x,c,:));
%    end
%  end
%end

%vectorized implementation
Ifftc_size = size( Ifftc );
Ifftc_size = Ifftc_size(1:3);

Ifftc = reshape ( Ifftc, prod( Ifftc_size ), num_tiles^2 )';

Ifftc = (M_hat_inv * Ifftc);
Ifftc = Ifftc';

Ifftc = reshape( Ifftc, Ifftc_size(1), Ifftc_size(2), ...
		 Ifftc_size(3), num_tiles^2 );

disp('inverse transform each of the individual tiles');
for y=1:num_tiles
  for x=1:num_tiles
    off = (y-1)*num_tiles+x;    
    for c=1:size(I,3)
      Ifftc(:,:,c,off) = abs(ifft2(ifftshift(squeeze(Ifftc(:,:,c,off)))));
    end
  end
end

disp('display results');
for i=1:num_tiles^2
    currentView = abs(Ifftc(:,:,:,i));
    imshow(squeeze( currentView ./ max(currentView(:)) ));
  drawnow
  pause(0.1)
end


