%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   Please check our website for the latest version of this script:
%   www.cs.ubc.ca/labs/imager/tr/2010/TheoryOfPlenopticMultiplexing/
%  
% general spatial reconstruction 
% applied to dappled photography 
% dataset 
%
% (c) Ivo Ihrke
%     University of British Columbia 
%
%  2009-04-20
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;

bShowIntermediateResults = 1;
bWriteOutput = 0;

filename = '../data/InputCones_9x9.png';

% check if file exists
if exist(filename, 'file') == 0
	error(['File ' filename ' does not exist. Make sure to follow the instructions in ../README!']);
end

%input image
I=double(imread(filename));

%sampling offset
I=circshift(I,[6 4]);

%dimensions
sx = 238;
sy = 191;

num_tiles = 9;

%start and end of up-sampled area
startx=1;
starty=1;
ex = (sx-1);
ey = (sy-1);


%up-sampling factor 
fx = 1/3;
fy = 1/3;

[Xi,Yi] = meshgrid(startx:fx:ex,starty:fy:ey);

Xi = Xi;
Yi = Yi;

%assemble low-res views

offx=( ones(num_tiles,1)*(0:(num_tiles-1)) ) / num_tiles;
offy=( ones(num_tiles,1)*(0:(num_tiles-1)) )' / num_tiles;

disp('assembling correlated views');
Ic = zeros( sy, sx, size(I,3), num_tiles^2 );
for y=1:num_tiles
  for x=1:num_tiles
    off = (y-1)*num_tiles + x; 
    Ic(:,:,:,off) = I(y:9:size(I,1),x:9:size(I,2),:);
  end
end

[X,Y] = meshgrid(1:size(Ic,2), 1:size(Ic,1));

disp('upsampling correlated views');

F=fspecial('sobel')';

Ic_up = zeros ( size(Yi,1), size(Yi,2), size(I,3), num_tiles^2 );
for y=1:num_tiles
  for x=1:num_tiles
    for c=1:size(I,3)
      off = (y-1)*num_tiles+x;
      disp(sprintf('%d:(%f,%f)',off,offx(y,x),offy(y,x)));
      Iintermed = interp2 ( X+offx(y,x), Y+offy(y,x), squeeze( Ic( :,:,c,off) ), Xi, Yi,'cubic');

% comment in for filter demo
%       Iintermed = imfilter( Iintermed, F );

       Ic_up(:,:,c,off) = Iintermed; 
    end
    if bShowIntermediateResults
        imshow(squeeze(Ic_up(:,:,:,off)/255));
        title('Upsampled Correlated Channels');
        drawnow
    end
  end
end

disp('assembling correlation matrix');

%assemble de-correlation matrix
F = zeros( num_tiles^2, num_tiles^2 );
S = complex(zeros( num_tiles^2, num_tiles^2 ));
for y=1:num_tiles
  for x=1:num_tiles
    off = (y-1)*num_tiles+x;
    dummy=zeros(num_tiles, num_tiles );
    dummys=zeros(num_tiles, num_tiles );
    dummy(y,x) = 1;
    dummys(y,x) = complex( 1, -1 )/2;
    dummys(num_tiles-y+1,num_tiles-x+1) = complex( 1, 1 )/2;
    

    dummy = ifft2(ifftshift(dummy));
    Finv( :, off ) = dummy(:); 
    S (:,off)  = dummys(:);
  end
end
% DC peak is real peak
S(floor(num_tiles^2/2)+1,floor(num_tiles^2/2)+1)=1;

%%% real basis 
%%%
%%% numerical problems below
%%%
%%% Finvreal = zeros(size(Finv));
%%% for i=1:floor(num_tiles^2 / 2)
%%%   off = i*2;
%%%   a   = Finv(:,i);
%%%   b   = Finv(:,num_tiles^2-(i-1));
%%% 
%%%   %real part (cos) - symmetric
%%%   Finvreal( :, off - 1 ) = real(a + b)/2;
%%% 
%%%   %imag part (sin) - antisymmetric
%%%   Finvreal( :, off     ) = imag(a - b)/2;
%%% end
%%% Finvreal( :, num_tiles^2) = Finv(:, floor(num_tiles^2 / 2)+1 );
%%% 

%%% stable version: use matrix S
Finvreal = real(Finv * S);
%%%

F=inv(Finv);

W = eye(num_tiles^2);
W(41,41)= 20;

% compute spatial and plenoptic correlation matrices

PI = F' * sqrt(W)';
SIGMA = Finv * sqrt(W);

Minv = pinv( PI' ) * pinv ( SIGMA ); 

disp('de-correlating up-sampled views');



% this is the conceptual loop
%Ic_up_dec = zeros( size ( Ic_up ) );
%for x=1:size(Ic_up,2)
%  for y=1:size(Ic_up,1)
%    for c=1:size(I,3)
%      Ic_up_dec( y, x, c, : ) = Minv * squeeze(Ic_up(y,x,c,:));
%    end
%  end
%end

%vectorized implementation
Ic_up_size = size( Ic_up );
Ic_up_size = Ic_up_size(1:3);

Ic_up = reshape ( Ic_up, prod( Ic_up_size ), num_tiles^2 )';


% invert plenoptic and spatial basis
Ic_up = (Minv * Ic_up);
Ic_up = Ic_up';

Ic_up = reshape( Ic_up, Ic_up_size(1), Ic_up_size(2), ...
		     Ic_up_size(3), num_tiles^2 );


Ic_up = Ic_up * 2;

if bWriteOutput
    [a b c] = mkdir('results');
end 

% visualize & write
for i=1:num_tiles^2
    if bShowIntermediateResults
        imshow(squeeze(real(Ic_up(:,:,:,i))/255)*4);
        title('Upsampled Decorrelated Channels');
        drawnow;
    end
    if bWriteOutput
    	imwrite(squeeze(real(Ic_up(:,:,:,i))/255)*4,sprintf('results/lf_spatial_cubic_%d_%d.png',mod(i,num_tiles),floor(i/num_tiles)));
    end
end

