%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   Please check our website for the latest version of this script:
%   www.cs.ubc.ca/labs/imager/tr/2010/TheoryOfPlenopticMultiplexing/
%  
%   Helper function for spatial reconstruction of Shield Fields datasets.
%
% (c) Ivo Ihrke
%     University of British Columbia 
%
%  2009-04-20
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function Iout = joint_bilateral_up_cubic_spatial ( X, Y, I, Iguide, xup, yup );


SIGMA = 0.01
EPS   = 0.01;
WEPS = 1*10e-01;

%bicubic

%inspired by CImg code
%
% modify to joint bilateral and fix bug
%

%sample positions
cx  = floor( xup );
px = cx - 1;
nx = cx + 1;
ax = cx + 2;

cy  = floor( yup );
py = cy - 1;
ny = cy + 1;
ay = cy + 2;

% boundary check

cx ( cx < 1 ) = 1;
cx ( cx > size ( I, 2 ) ) = size ( I, 2 );

px ( px < 1 ) = 1;
px ( px > size ( I, 2 ) ) = size ( I, 2 );

nx ( nx < 1 ) = 1;
nx ( nx > size ( I, 2 ) ) = size ( I, 2 );

ax ( ax < 1 ) = 1;
ax ( ax > size ( I, 2 ) ) = size ( I, 2 );


cy ( cy < 1 ) = 1;
cy ( cy > size ( I, 1 ) ) = size ( I, 1 );

py ( py < 1 ) = 1;
py ( py > size ( I, 1 ) ) = size ( I, 1 );

ny ( ny < 1 ) = 1;
ny ( ny > size ( I, 1 ) ) = size ( I, 1 );

ay ( ay < 1 ) = 1;
ay ( ay > size ( I, 1 ) ) = size ( I, 1 );


%sample original image at shifted positions

Ipp = reshape(I( sub2ind ( size(I), py, px ) ), size(xup,1), size(xup,2) );
Ipc = reshape(I( sub2ind ( size(I), py, cx ) ), size(xup,1), size(xup,2) );
Ipn = reshape(I( sub2ind ( size(I), py, nx ) ), size(xup,1), size(xup,2) );
Ipa = reshape(I( sub2ind ( size(I), py, ax ) ), size(xup,1), size(xup,2) );

Icp = reshape(I( sub2ind ( size(I), cy, px ) ), size(xup,1), size(xup,2) );
Icc = reshape(I( sub2ind ( size(I), cy, cx ) ), size(xup,1), size(xup,2) );
Icn = reshape(I( sub2ind ( size(I), cy, nx ) ), size(xup,1), size(xup,2) );
Ica = reshape(I( sub2ind ( size(I), cy, ax ) ), size(xup,1), size(xup,2) );

Inp = reshape(I( sub2ind ( size(I), ny, px ) ), size(xup,1), size(xup,2) );
Inc = reshape(I( sub2ind ( size(I), ny, cx ) ), size(xup,1), size(xup,2) );
Inn = reshape(I( sub2ind ( size(I), ny, nx ) ), size(xup,1), size(xup,2) );
Ina = reshape(I( sub2ind ( size(I), ny, ax ) ), size(xup,1), size(xup,2) );

Iap = reshape(I( sub2ind ( size(I), ay, px ) ), size(xup,1), size(xup,2) );
Iac = reshape(I( sub2ind ( size(I), ay, cx ) ), size(xup,1), size(xup,2) );
Ian = reshape(I( sub2ind ( size(I), ay, nx ) ), size(xup,1), size(xup,2) );
Iaa = reshape(I( sub2ind ( size(I), ay, ax ) ), size(xup,1), size(xup,2) );


%also need positions in guide image (different resolution, 
%preferably higher :)

facx = size(Iguide,2) / size(I,2);	

%floating point pos.
xupg = xup * facx;

%integer positions surrounding fl.p. pos.
pxg = px * facx;   
cxg = cx * facx;   
nxg = nx * facx;   
axg = ax * facx;   


facy = size(Iguide,1) / size(I,1);	

%floating point pos.
yupg = yup * facy;

%integer positions surrounding fl.p. pos.
pyg = py * facy;   
cyg = cy * facy;   
nyg = ny * facy;   
ayg = ay * facy;   


% sample guide image to compute weights
% (distance in intensity space )
%
[X,Y] = meshgrid( 1:size(Iguide,2), 1:size(Iguide,1) );



gI   = interp2( X, Y, Iguide, xupg, yupg );

gIpp = interp2( X, Y, Iguide, pxg, pyg );
gIpc = interp2( X, Y, Iguide, cxg, pyg );
gIpn = interp2( X, Y, Iguide, nxg, pyg );
gIpa = interp2( X, Y, Iguide, axg, pyg );

gIcp = interp2( X, Y, Iguide, pxg, cyg );
gIcc = interp2( X, Y, Iguide, cxg, cyg );
gIcn = interp2( X, Y, Iguide, nxg, cyg );
gIca = interp2( X, Y, Iguide, axg, cyg );

gInp = interp2( X, Y, Iguide, pxg, nyg );
gInc = interp2( X, Y, Iguide, cxg, nyg );
gInn = interp2( X, Y, Iguide, nxg, nyg );
gIna = interp2( X, Y, Iguide, axg, nyg );

gIap = interp2( X, Y, Iguide, pxg, ayg );
gIac = interp2( X, Y, Iguide, cxg, ayg );
gIan = interp2( X, Y, Iguide, nxg, ayg );
gIaa = interp2( X, Y, Iguide, axg, ayg );

%% weights
wpp  = exp ( - ( gI - gIpp ).^2 / SIGMA^2 ) + EPS;
wpc  = exp ( - ( gI - gIpc ).^2 / SIGMA^2 ) + EPS;
wpn  = exp ( - ( gI - gIpn ).^2 / SIGMA^2 ) + EPS;
wpa  = exp ( - ( gI - gIpa ).^2 / SIGMA^2 ) + EPS;

wcp  = exp ( - ( gI - gIcp ).^2 / SIGMA^2 ) + EPS;
wcc  = exp ( - ( gI - gIcc ).^2 / SIGMA^2 ) + EPS;
wcn  = exp ( - ( gI - gIcn ).^2 / SIGMA^2 ) + EPS;
wca  = exp ( - ( gI - gIca ).^2 / SIGMA^2 ) + EPS;

wnp  = exp ( - ( gI - gInp ).^2 / SIGMA^2 ) + EPS;
wnc  = exp ( - ( gI - gInc ).^2 / SIGMA^2 ) + EPS;
wnn  = exp ( - ( gI - gInn ).^2 / SIGMA^2 ) + EPS;
wna  = exp ( - ( gI - gIna ).^2 / SIGMA^2 ) + EPS;

wap  = exp ( - ( gI - gIap ).^2 / SIGMA^2 ) + EPS;
wac  = exp ( - ( gI - gIac ).^2 / SIGMA^2 ) + EPS;
wan  = exp ( - ( gI - gIan ).^2 / SIGMA^2 ) + EPS;
waa  = exp ( - ( gI - gIaa ).^2 / SIGMA^2 ) + EPS;





%% weigh the images 

Ipp = Ipp .* wpp;
Ipc = Ipc .* wpc;
Ipn = Ipn .* wpn;
Ipa = Ipa .* wpa;

Icp = Icp .* wcp;
Icc = Icc .* wcc;
Icn = Icn .* wcn;
Ica = Ica .* wca;

Inp = Inp .* wnp;
Inc = Inc .* wnc;
Inn = Inn .* wnn;
Ina = Ina .* wna;

Iap = Iap .* wap;
Iac = Iac .* wac;
Ian = Ian .* wan;
Iaa = Iaa .* waa;


dx  = xup - cx;
dx2 = dx  .* dx;
dx3 = dx2 .* dx;

dy =  yup - cy;
dy2 = dy  .* dy;
dy3 = dy2 .* dy;


FIX = 0;

%% compute 4 x-interpolated values

    % y - dy (at yp)

    %weighted image
    u0p = Ipc - Ipp;  %gradient left
    u1p = Ipa - Ipn;  %gradient right
     ap = -1 * Ipp + 3 * Ipc - 3 * Ipn + 1 * Ipa;   %third derivative
     bp =  2 * Ipp - 5 * Ipc + 4 * Ipn - 1 * Ipa;   %second - third derivative 
    
    if ( FIX )
     %this is a bug-fix compared to CImg 
     bp = bp + ap; % (remove negative third derivative term from bp)
    end

    valp = ap .* dx3 + bp .* dx2  + u0p .* dx + Ipc;

    %sum of weights
    u0p = wpc - wpp;  %gradient left
    u1p = wpa - wpn;  %gradient right
     ap = -1 * wpp + 3 * wpc - 3 * wpn + 1 * wpa;   %third derivative
     bp =  2 * wpp - 5 * wpc + 4 * wpn - 1 * wpa;   %second - third derivative 
    
    if ( FIX )
     %this is a bug-fix compared to CImg 
     bp = bp + ap; % (remove negative third derivative term from bp)
    end

    valwp = ap .* dx3 + bp .* dx2  + u0p .* dx + wpc;


    % y    (at yc)

    %weighted image
    u0c = Icc - Icp;  %gradient left
    u1c = Ica - Icn;  %gradient right
     ac = -1 * Icp + 3 * Icc - 3 * Icn + 1 * Ica;   %third derivative
     bc =  2 * Icp - 5 * Icc + 4 * Icn - 1 * Ica;   %second - third derivative 

    if ( FIX )    
     %this is a bug-fix compared to CImg 
     bc = bc + ac; % (remove negative third derivative term from bp)
    end

    valc = ac .* dx3 + bc .* dx2  + u0c .* dx + Icc;

    %sum of weights
    u0c = wcc - wcp;  %gradient left
    u1c = wca - wcn;  %gradient right
     ac = -1 * wcp + 3 * wcc - 3 * wcn + 1 * wca;   %third derivative
     bc =  2 * wcp - 5 * wcc + 4 * wcn - 1 * wca;   %second - third derivative 

    if ( FIX )    
     %this is a bug-fix compared to CImg 
     bc = bc + ac; % (remove negative third derivative term from bp)
    end

    valwc = ac .* dx3 + bc .* dx2  + u0c .* dx + wcc;


    % y + dy (at yn)

    %weighted image
    u0n = Inc - Inp;  %gradient left
    u1n = Ina - Inn;  %gradient right
     an = -1 * Inp + 3 * Inc - 3 * Inn + 1 * Ina;   %third derivative
     bn =  2 * Inp - 5 * Inc + 4 * Inn - 1 * Ina;   %second - third derivative 

    if ( FIX )    
     %this is a bug-fix compared to CImg 
     bn = bn + an; % (remove negative third derivative term from bp)
    end

    valn = an .* dx3 + bn .* dx2  + u0n .* dx + Inc;

    %sum of weights
    u0n = wnc - wnp;  %gradient left
    u1n = wna - wnn;  %gradient right
     an = -1 * wnp + 3 * wnc - 3 * wnn + 1 * wna;   %third derivative
     bn =  2 * wnp - 5 * wnc + 4 * wnn - 1 * wna;   %second - third derivative 

    if ( FIX )    
     %this is a bug-fix compared to CImg 
     bn = bn + an; % (remove negative third derivative term from bp)
    end

    valwn = an .* dx3 + bn .* dx2  + u0n .* dx + wnc;



    % y + 2 * dy (at ya)

    %weighted image
    u0a = Iac - Iap;  %gradient left
    u1a = Iaa - Ian;  %gradient right
     aa = -1 * Iap + 3 * Iac - 3 * Ian + 1 * Iaa;   %third derivative
     ba =  2 * Iap - 5 * Iac + 4 * Ian - 1 * Iaa;   %second - third derivative 

    if ( FIX )    
     %this is a bug-fix compared to CImg 
     ba = ba + aa; % (remove negative third derivative term from bp)
    end      

    vala = aa .* dx3 + ba .* dx2  + u0a .* dx + Iac;

    %sum of weights
    u0a = wac - wap;  %gradient left
    u1a = waa - wan;  %gradient right
     aa = -1 * wap + 3 * wac - 3 * wan + 1 * waa;   %third derivative
     ba =  2 * wap - 5 * wac + 4 * wan - 1 * waa;   %second - third derivative 
    
    if ( FIX )
     %this is a bug-fix compared to CImg 
     ba = ba + aa; % (remove negative third derivative term from bp)
    end

    valwa = aa .* dx3 + ba .* dx2  + u0a .* dx + wac;



%% final cubic y - interpolation


u0 = valc - valp;  %gradient left
u1 = vala - valn;  %gradient right
 a = -1 * valp + 3 * valc - 3 * valn + 1 * vala;   %third derivative
 b =  2 * valp - 5 * valc + 4 * valn - 1 * vala;   %second - third derivative 

if ( FIX )    
 %this is a bug-fix compared to CImg 
 b  = b + a; % (remove negative third derivative term from bp)
end   

Iout = a .* dy3 + b .* dy2  + u0 .* dy + valc;


%sum of weights

u0 = valwc - valwp;  %gradient left
u1 = valwa - valwn;  %gradient right
 a = -1 * valwp + 3 * valwc - 3 * valwn + 1 * valwa;   %third derivative
 b =  2 * valwp - 5 * valwc + 4 * valwn - 1 * valwa;   %second - third derivative 

if ( FIX )    
 %this is a bug-fix compared to CImg 
 b  = b + a; % (remove negative third derivative term from bp)
end
   
Ioutw = a .* dy3 + b .* dy2  + u0 .* dy + valwc;




%Iout( Iout < 0 ) = 0;
%min(Ioutw(:))

%min(Iout(:))

%Ioutw ( Ioutw < WEPS ) = WEPS;

%Ioutw ( Ioutw < WEPS ) = WEPS;

Iout = Iout ./ Ioutw;

Iout(Iout>255) = 255;
Iout(Iout < 0 ) = 0 ;

%Iout = Iout ./ Ioutw;

%Iout = vala;

