%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   Please check our website for the latest version of this script:
%   www.cs.ubc.ca/labs/imager/tr/2010/TheoryOfPlenopticMultiplexing/
%
%   This script processes the original golgi strained neuron to be used
%   with our noise analysis scripts.
%
%   You need to download the light field from 
%       http://lightfield.stanford.edu/data/golgi40x/golgi40x_interleaved_square_autorot.jpg
%
%   Gordon Wetzstein [wetzste1@cs.ubc.ca]
%   PSM Lab, UBC
%   March 2010
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;

% the light field
filename = 'golgi40x_interleaved_square_autorot.jpg';

% check if file exists
if exist(filename, 'file') == 0
    error('You need to download the file http://lightfield.stanford.edu/data/golgi40x/golgi40x_interleaved_square_autorot.jpg and save it in the data folder!');
end

% load light field
LF = double(imread(filename)) ./ 255;

% crop black boarders
x=3*16; y=16*50; LF = mean(LF(y:y+2992-1, x:x+4528-1, :), 3);

% number of views in original light field and in resampled one
numDirs     = [16 16]; 
numDirsNew  = [11 11];

% number of lenslets
numLenslets = [size(LF,1)/numDirs(1) size(LF,2)/numDirs(2)];

% resample to 11x11 views
LFnew = zeros([numDirsNew(1)*numLenslets(1) numDirsNew(2)*numLenslets(2)]);
for ly=1:numLenslets(1)
    for lx=1:numLenslets(2)
        lenslet = LF(   (ly-1)*numDirs(1)+1:ly*numDirs(1), ...
                        (lx-1)*numDirs(2)+1:lx*numDirs(2), :);
                    
        LFnew(  (ly-1)*numDirsNew(1)+1:ly*numDirsNew(1), ...
                (lx-1)*numDirsNew(2)+1:lx*numDirsNew(2), :) = imresize(lenslet, numDirsNew);
    end
end

imwrite(LFnew, 'Golgi11x11.png');
LFnew = lightFieldSpatialMajor2DirectionMajor(LFnew, numLenslets);
imwrite(LFnew, 'Golgi11x11DM.png');